@@ -1,6 +1,6 @@
 /*
  *  git2r, R bindings to the libgit2 library.
- *  Copyright (C) 2013-2015 The git2r contributors
+ *  Copyright (C) 2013-2018 The git2r contributors
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License, version 2,
@@ -16,14 +16,13 @@
  *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
-#include <Rdefines.h>
-#include "git2.h"
-#include "cc-compat.h"
+#include <git2.h>
 
 #include "git2r_arg.h"
 #include "git2r_commit.h"
 #include "git2r_error.h"
 #include "git2r_repository.h"
+#include "git2r_S3.h"
 #include "git2r_signature.h"
 #include "git2r_stash.h"
 
@@ -39,15 +38,15 @@ typedef struct {
 } git2r_stash_list_cb_data;
 
 /**
- * Remove a stash from the stash list
+ * Apply a single stashed state from the stash list.
  *
- * @param repo S4 class git_repository that contains the stash
+ * @param repo S3 class git_repository that contains the stash
  * @param index The index to the stash. 0 is the most recent stash.
  * @return R_NilValue
  */
-SEXP git2r_stash_drop(SEXP repo, SEXP index)
+SEXP git2r_stash_apply(SEXP repo, SEXP index)
 {
-    int err;
+    int error;
     git_repository *repository = NULL;
 
     if (git2r_arg_check_integer_gte_zero(index))
@@ -57,21 +56,80 @@ SEXP git2r_stash_drop(SEXP repo, SEXP index)
     if (!repository)
         git2r_error(__func__, NULL, git2r_err_invalid_repository, NULL);
 
-    err = git_stash_drop(repository, INTEGER(index)[0]);
+    error = git_stash_apply(repository, INTEGER(index)[0], NULL);
+    if (error == GIT_ENOTFOUND)
+        error = 0;
     git_repository_free(repository);
-    if (err)
+    if (error)
         git2r_error(__func__, giterr_last(), NULL, NULL);
 
     return R_NilValue;
 }
 
 /**
- * Init slots in S4 class git_stash
+ * Remove a stash from the stash list
+ *
+ * @param repo S3 class git_repository that contains the stash
+ * @param index The index to the stash. 0 is the most recent stash.
+ * @return R_NilValue
+ */
+SEXP git2r_stash_drop(SEXP repo, SEXP index)
+{
+    int error;
+    git_repository *repository = NULL;
+
+    if (git2r_arg_check_integer_gte_zero(index))
+        git2r_error(__func__, NULL, "'index'", git2r_err_integer_gte_zero_arg);
+
+    repository = git2r_repository_open(repo);
+    if (!repository)
+        git2r_error(__func__, NULL, git2r_err_invalid_repository, NULL);
+
+    error = git_stash_drop(repository, INTEGER(index)[0]);
+    git_repository_free(repository);
+    if (error)
+        git2r_error(__func__, giterr_last(), NULL, NULL);
+
+    return R_NilValue;
+}
+
+/**
+ * Apply a single stashed state from the stash list and remove it from
+ * the list if successful.
+ *
+ * @param repo S3 class git_repository that contains the stash
+ * @param index The index to the stash. 0 is the most recent stash.
+ * @return R_NilValue
+ */
+SEXP git2r_stash_pop(SEXP repo, SEXP index)
+{
+    int error;
+    git_repository *repository = NULL;
+
+    if (git2r_arg_check_integer_gte_zero(index))
+        git2r_error(__func__, NULL, "'index'", git2r_err_integer_gte_zero_arg);
+
+    repository = git2r_repository_open(repo);
+    if (!repository)
+        git2r_error(__func__, NULL, git2r_err_invalid_repository, NULL);
+
+    error = git_stash_pop(repository, INTEGER(index)[0], NULL);
+    if (error == GIT_ENOTFOUND)
+        error = 0;
+    git_repository_free(repository);
+    if (error)
+        git2r_error(__func__, giterr_last(), NULL, NULL);
+
+    return R_NilValue;
+}
+
+/**
+ * Init slots in S3 class git_stash
  *
  * @param source The commit oid of the stashed state.
  * @param repository The repository
- * @param repo S4 class git_repository that contains the stash
- * @param dest S4 class git_stash to initialize
+ * @param repo S3 class git_repository that contains the stash
+ * @param dest S3 class git_stash to initialize
  * @return int 0 on success, or an error code.
  */
 int git2r_stash_init(
@@ -80,12 +138,12 @@ int git2r_stash_init(
     SEXP repo,
     SEXP dest)
 {
-    int err;
+    int error;
     git_commit *commit = NULL;
 
-    err = git_commit_lookup(&commit, repository, source);
-    if (err)
-        return err;
+    error = git_commit_lookup(&commit, repository, source);
+    if (error)
+        return error;
     git2r_commit_init(commit, repo, dest);
     git_commit_free(commit);
 
@@ -110,25 +168,26 @@ static int git2r_stash_list_cb(
 {
     git2r_stash_list_cb_data *cb_data = (git2r_stash_list_cb_data*)payload;
 
-    GIT_UNUSED(index);
-    GIT_UNUSED(message);
-
     /* Check if we have a list to populate */
-    if (R_NilValue != cb_data->list) {
-        int err;
-        SEXP stash;
+    if (!Rf_isNull(cb_data->list)) {
+        int error;
+        SEXP stash, class;
 
         SET_VECTOR_ELT(
             cb_data->list,
             cb_data->n,
-            stash = NEW_OBJECT(MAKE_CLASS("git_stash")));
-        err = git2r_stash_init(
+            stash = Rf_mkNamed(VECSXP, git2r_S3_items__git_commit));
+        Rf_setAttrib(stash, R_ClassSymbol, class = Rf_allocVector(STRSXP, 2));
+        SET_STRING_ELT(class, 0, Rf_mkChar("git_stash"));
+        SET_STRING_ELT(class, 1, Rf_mkChar("git_commit"));
+
+        error = git2r_stash_init(
             stash_id,
             cb_data->repository,
             cb_data->repo,
             stash);
-        if (err)
-            return err;
+        if (error)
+            return error;
     }
 
     cb_data->n += 1;
@@ -139,13 +198,13 @@ static int git2r_stash_list_cb(
 /**
  * List stashes in a repository
  *
- * @param repo S4 class git_repository
- * @return VECXSP with S4 objects of class git_stash
+ * @param repo S3 class git_repository
+ * @return VECXSP with S3 objects of class git_stash
  */
 SEXP git2r_stash_list(SEXP repo)
 {
     SEXP list = R_NilValue;
-    int err;
+    int error, nprotect = 0;
     git2r_stash_list_cb_data cb_data = {0, R_NilValue, R_NilValue, NULL};
     git_repository *repository = NULL;
 
@@ -154,25 +213,25 @@ SEXP git2r_stash_list(SEXP repo)
         git2r_error(__func__, NULL, git2r_err_invalid_repository, NULL);
 
     /* Count number of stashes before creating the list */
-    err = git_stash_foreach(repository, &git2r_stash_list_cb, &cb_data);
-    if (err)
+    error = git_stash_foreach(repository, &git2r_stash_list_cb, &cb_data);
+    if (error)
         goto cleanup;
 
-    PROTECT(list = allocVector(VECSXP, cb_data.n));
+    PROTECT(list = Rf_allocVector(VECSXP, cb_data.n));
+    nprotect++;
     cb_data.n = 0;
     cb_data.list = list;
     cb_data.repo = repo;
     cb_data.repository = repository;
-    err = git_stash_foreach(repository, &git2r_stash_list_cb, &cb_data);
+    error = git_stash_foreach(repository, &git2r_stash_list_cb, &cb_data);
 
 cleanup:
-    if (repository)
-        git_repository_free(repository);
+    git_repository_free(repository);
 
-    if (R_NilValue != list)
-        UNPROTECT(1);
+    if (nprotect)
+        UNPROTECT(nprotect);
 
-    if (err)
+    if (error)
         git2r_error(__func__, giterr_last(), NULL, NULL);
 
     return list;
@@ -190,7 +249,7 @@ cleanup:
  * @param ignored All ignored files are also stashed and then cleaned
  *        up from the working directory. Default is FALSE
  * @param stasher Signature with stasher and time of stash
- * @return S4 class git_stash
+ * @return S3 class git_stash
  */
 SEXP git2r_stash_save(
     SEXP repo,
@@ -200,8 +259,8 @@ SEXP git2r_stash_save(
     SEXP ignored,
     SEXP stasher)
 {
-    int err;
-    SEXP result = R_NilValue;
+    int error, nprotect = 0;
+    SEXP result = R_NilValue, class;
     git_oid oid;
     git_stash_flags flags = GIT_STASH_DEFAULT;
     git_commit *commit = NULL;
@@ -228,39 +287,38 @@ SEXP git2r_stash_save(
     if (LOGICAL(ignored)[0])
         flags |= GIT_STASH_INCLUDE_IGNORED;
 
-    err = git2r_signature_from_arg(&c_stasher, stasher);
-    if (err)
+    error = git2r_signature_from_arg(&c_stasher, stasher);
+    if (error)
         goto cleanup;
 
-    err = git_stash_save(
+    error = git_stash_save(
         &oid,
         repository,
         c_stasher,
         CHAR(STRING_ELT(message, 0)),
         flags);
-    if (err) {
-        if (GIT_ENOTFOUND == err)
-            err = GIT_OK;
+    if (error) {
+        if (GIT_ENOTFOUND == error)
+            error = GIT_OK;
         goto cleanup;
     }
 
-    PROTECT(result = NEW_OBJECT(MAKE_CLASS("git_stash")));
-    err = git2r_stash_init(&oid, repository, repo, result);
+    PROTECT(result = Rf_mkNamed(VECSXP, git2r_S3_items__git_commit));
+    nprotect++;
+    Rf_setAttrib(result, R_ClassSymbol, class = Rf_allocVector(STRSXP, 2));
+    SET_STRING_ELT(class, 0, Rf_mkChar("git_stash"));
+    SET_STRING_ELT(class, 1, Rf_mkChar("git_commit"));
+    error = git2r_stash_init(&oid, repository, repo, result);
 
 cleanup:
-    if (commit)
-        git_commit_free(commit);
+    git_commit_free(commit);
+    git_signature_free(c_stasher);
+    git_repository_free(repository);
 
-    if (c_stasher)
-        git_signature_free(c_stasher);
+    if (nprotect)
+        UNPROTECT(nprotect);
 
-    if (repository)
-        git_repository_free(repository);
-
-    if (R_NilValue != result)
-        UNPROTECT(1);
-
-    if (err)
+    if (error)
         git2r_error(__func__, giterr_last(), NULL, NULL);
 
     return result;
