@@ -14,7 +14,10 @@
 ## with this program; if not, write to the Free Software Foundation, Inc.,
 ## 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
-library(git2r)
+library("git2r")
+
+## For debugging
+sessionInfo()
 
 ## Create a directory in tempdir
 path <- tempfile(pattern="git2r-")
